#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Terrain" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

import sys

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from PIL import Image

from terrain import *


class myGLCanvas(object):
    def __init__(self):
        self.init = False        
        self._angle = 60.

        glutInit(sys.argv)
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        glutInitWindowSize(400, 400)
        glutCreateWindow(b"Terrain - videotutorialsrock.com")
        
        self.initRendering()
        self.loadTerrain()
 
        glutTimerFunc(25, self.update, 0)
        
        glutDisplayFunc(self.OnPaint)
        glutReshapeFunc(self.OnResize)
        glutKeyboardFunc(self.handleKeypress)

        glutMainLoop()
        return 0
        

    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_NORMALIZE)
        glShadeModel(GL_SMOOTH)
        
        
    def OnPaint(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glTranslatef(0.0, 0.0, -10.0)
        glRotatef(30.0, 1.0, 0.0, 0.0)
        glRotatef(-self._angle, 0.0, 1.0, 0.0)

        ambientColor = [0.4, 0.4, 0.4, 1.0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientColor)

        lightColor0 = [0.6, 0.6, 0.6, 1.0]
        lightPos0 = [-0.5, 0.8, 0.1, 0.0]
        glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor0)
        glLightfv(GL_LIGHT0, GL_POSITION, lightPos0)

        imgW = self.terrain.getWidth()
        imgL = self.terrain.getLength()

        scale = float(5./max(imgW - 1, imgL - 1))
        glScalef(scale, scale, scale)
        glTranslatef(-float(imgW - 1)/2., 0.0, -float(imgL - 1)/2.)
        
        glColor3f(0.3, 0.9, 0.0)
        for z in range (imgL-1) :
            glBegin(GL_TRIANGLE_STRIP)
            for x in range (imgW) :
                normal = self.terrain.getNormal(x, z)
                glNormal3f(normal[0], normal[1], normal[2])
                glVertex3f(x, self.terrain.getHight(x, z), z)
                normal = self.terrain.getNormal(x, z+1)
                glNormal3f(normal[0], normal[1], normal[2])
                glVertex3f(x, self.terrain.getHight(x, z+1), z+1)
            glEnd()

        glutSwapBuffers()
          

    def OnResize(self, w, h):
        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)
        
    
    def update(self, value):
        self._angle += 1.
        if self._angle > 360. : self._angle -= 360.
        
        glutPostRedisplay()
        glutTimerFunc(25, self.update, 0)
        

    def handleKeypress(self, key, x, y):
        if ord(key) == 27 :
            self.cleanup()
            exit()
      

    def cleanup(self):
        del self.terrain


    def loadTerrain(self, hight=20):
        image = Image.open("heightmap.bmp").transpose(Image.FLIP_TOP_BOTTOM)
        self.terrain = Terrain(image, hight)

        del image


if __name__ == '__main__':
    canvas = myGLCanvas()
