#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Terrain" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

from vecteur import *


class Terrain(object):
    def __init__(self, img, hight):
        self.width, self.length = img.size
        self.img = img
        self.hightValue = hight

        self.hight = {}
        self.normals = {}

        self.normalReady = False
        self.computeHight()
        self.computeNormals()


    def getWidth(self):
        return self.width


    def getLength(self):
        return self.length


    def computeHight(self):
        for z in range (self.length) :
            for x in range (self.width) :
                color = self.img.getpixel((x, z))[2]
                h = self.hightValue*((color/255.0) - 0.5)
                self.hight[(x,z)] = h
        del self.img


    def getHight(self, x, z):
        return self.hight[(x,z)]


    def getNormal(self, x, z):
        return self.normals[(x, z)].getCoord()


    def computeNormals(self):
        if self.normalReady : return

        normals2 = {}
        for z in range(self.length) :
            for x in range (self.width) :
                total = Vecteur(0., 0., 0.)
                
                if z > 0 :
                    v_out = Vecteur(0., \
                                    self.hight[(x,z-1)]-self.hight[(x,z)], \
                                    -1.)

                if z < self.length - 1 :
                    v_in = Vecteur(0., \
                                   self.hight[(x,z+1)]-self.hight[(x,z)], \
                                   1.)
                    
                if x > 0 :
                    v_left = Vecteur(-1., \
                                     self.hight[(x-1,z)]-self.hight[(x,z)], \
                                     0.)

                if x < self.width - 1 :
                    v_right = Vecteur(1., \
                                     self.hight[(x+1,z)]-self.hight[(x,z)], \
                                     0.)

                if (x > 0 and z > 0) :
                    total += v_out.vectoriel(v_left).unitaire()

                if (x > 0 and z < self.length - 1) :
                    total += v_left.vectoriel(v_in).unitaire()  

                if (x < self.width - 1 and z < self.length - 1) :
                    total += v_in.vectoriel(v_right).unitaire()

                if (x < self.width - 1 and z > 0) :
                    total += v_right.vectoriel(v_out).unitaire()

                normals2[(x,z)] = total

        fallout_ratio = 0.5
        for z in range(self.length) :
            for x in range (self.width) :
                total = normals2[(x,z)]

                if x > 0 :
                    total += normals2[(x-1,z)] * fallout_ratio

                if x < self.width - 1 :
                    total += normals2[(x+1,z)] * fallout_ratio

                if z > 0 :
                    total += normals2[(x,z-1)]* fallout_ratio

                if z < self.length -1 :
                    total += normals2[(x,z+1)] * fallout_ratio

                if total.norme() == 0 :
                    total = Vecteur(0.0, 1.0, 0.0)
                self.normals[(x,z)] = total

        del normals2

        self.normalReady = True
